'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var ramda = require('ramda');

/**
 * Performs a breadth-first traversal over a set of states.
 * Starting with `initial`, and generating neighboring states with `next`.
 * This generator yields each state as it is visited.
 * Caution: If your states grow infinitely, so will this generator
 *
 * @param next
 * @param start
 */
const breadthFirstTraversal = function* (next, start) {
    const visited = new Set();
    // we queue a pair of values and the path through to get there
    const queue = [[start, []]];
    while (queue.length) {
        const [value, pathSoFar] = queue.shift();
        const asStr = ramda.toString(value);
        if (visited.has(asStr))
            continue;
        yield [value, [...pathSoFar, value]];
        visited.add(asStr);
        const nextPathSoFar = [...pathSoFar, value];
        queue.push(...next(value)
            .filter(v => !visited.has(ramda.toString(v)))
            .map(v => [v, nextPathSoFar]));
    }
};
/**
 * Performs a breadth-first-search (bfs) over a set of states.
 * Starting with `initial`, and generating neighboring states with `next`
 * Returns a path to a state when `found` returns `true`
 * Returns `undefined` if no path is possible.
 *
 * @public
 * @param next - Function to generate "next" states given a current state
 * @param found - Predicate to determine if solution found. `bfs` returns a path to the first state for which this predicate returns `true`.
 * @param start - Initial state
 * @returns First path found to a state matching the predicate, or `null` if no such path exists.
 */
const breadthFirstSearch = (next, found, start) => {
    const visited = [];
    for (const [value, pathTo] of breadthFirstTraversal(next, start)) {
        visited.push(value);
        if (found(value))
            return [value, pathTo, visited];
    }
    return undefined;
};

exports.breadthFirstSearch = breadthFirstSearch;
exports.breadthFirstTraversal = breadthFirstTraversal;
//# sourceMappingURL=breadthFirst.cjs.map
